@@ -150,7 +150,7 @@ w_err(SerdWriter* writer, SerdStatus st, const char* fmt, ...)
   va_end(args);
 }
 
-static inline WriteContext*
+SERD_PURE_FUNC static WriteContext*
 anon_stack_top(SerdWriter* writer)
 {
   assert(!serd_stack_is_empty(&writer->anon_stack));
@@ -173,7 +173,7 @@ copy_node(SerdNode* dst, const SerdNode* src)
   }
 }
 
-static inline size_t
+static size_t
 sink(const void* buf, size_t len, SerdWriter* writer)
 {
   return serd_byte_sink_write(buf, len, &writer->byte_sink);
@@ -188,8 +188,8 @@ write_character(SerdWriter* writer, const uint8_t* utf8, size_t* size)
   const uint32_t c          = parse_utf8_char(utf8, size);
   switch (*size) {
   case 0:
-    w_err(writer, SERD_ERR_BAD_ARG, "invalid UTF-8: %X\n", utf8[0]);
-    return sink(replacement_char, sizeof(replacement_char), writer);
+    w_err(writer, SERD_ERR_BAD_ARG, "invalid UTF-8 start: %X\n", utf8[0]);
+    return 0;
   case 1:
     snprintf(escape, sizeof(escape), "\\u%04X", utf8[0]);
     return sink(escape, 6, writer);
@@ -211,7 +211,7 @@ write_character(SerdWriter* writer, const uint8_t* utf8, size_t* size)
   return sink(escape, 10, writer);
 }
 
-static inline bool
+static bool
 uri_must_escape(const uint8_t c)
 {
   switch (c) {
@@ -254,8 +254,11 @@ write_uri(SerdWriter* writer, const uint8_t* utf8, size_t n_bytes)
     len += write_character(writer, utf8 + i, &size);
     i += size;
     if (size == 0) {
-      // Corrupt input, scan to start of next character
-      for (++i; i < n_bytes && (utf8[i] & 0x80); ++i) {
+      // Corrupt input, write percent-encoded bytes and scan to next start
+      char escape[4] = {0, 0, 0, 0};
+      for (; i < n_bytes && (utf8[i] & 0x80); ++i) {
+        snprintf(escape, sizeof(escape), "%%%02X", utf8[i]);
+        len += sink(escape, 3, writer);
       }
     }
   }
@@ -332,8 +335,13 @@ write_text(SerdWriter*    writer,
            const uint8_t* utf8,
            size_t         n_bytes)
 {
-  size_t len = 0;
+  size_t len                  = 0;
+  size_t n_consecutive_quotes = 0;
   for (size_t i = 0; i < n_bytes;) {
+    if (utf8[i] != '"') {
+      n_consecutive_quotes = 0;
+    }
+
     // Fast bulk write for long strings of printable ASCII
     size_t j = i;
     for (; j < n_bytes; ++j) {
@@ -350,6 +358,8 @@ write_text(SerdWriter*    writer,
 
     const uint8_t in = utf8[i++];
     if (ctx == WRITE_LONG_STRING) {
+      n_consecutive_quotes = (in == '\"') ? (n_consecutive_quotes + 1) : 0;
+
       switch (in) {
       case '\\':
         len += sink("\\\\", 2, writer);
@@ -364,7 +374,8 @@ write_text(SerdWriter*    writer,
         len += sink(&in, 1, writer); // Write character as-is
         continue;
       case '\"':
-        if (i == n_bytes) { // '"' at string end
+        if (n_consecutive_quotes >= 3 || i == n_bytes) {
+          // Two quotes in a row, or quote at string end, escape
           len += sink("\\\"", 2, writer);
         } else {
           len += sink(&in, 1, writer);
@@ -411,7 +422,8 @@ write_text(SerdWriter*    writer,
     size_t size = 0;
     len += write_character(writer, utf8 + i - 1, &size);
     if (size == 0) {
-      // Corrupt input, scan to start of next character
+      // Corrupt input, write replacement character and scan to the next start
+      len += sink(replacement_char, sizeof(replacement_char), writer);
       for (; i < n_bytes && (utf8[i] & 0x80); ++i) {
       }
     } else {
@@ -539,8 +551,8 @@ write_literal(SerdWriter*        writer,
   return true;
 }
 
-// Return true iff `buf` is a valid prefixed name suffix
-static inline bool
+// Return true iff `buf` is a valid prefixed name prefix or suffix
+static bool
 is_name(const uint8_t* buf, const size_t len)
 {
   // TODO: This is more strict than it should be
@@ -581,6 +593,7 @@ write_uri_node(SerdWriter* const        writer,
 
     if (has_scheme && (writer->style & SERD_STYLE_CURIED) &&
         serd_env_qualify(writer->env, node, &prefix, &suffix) &&
+        is_name(prefix.buf, prefix.n_bytes) &&
         is_name(suffix.buf, suffix.len)) {
       write_uri(writer, prefix.buf, prefix.n_bytes);
       sink(":", 1, writer);
@@ -743,7 +756,7 @@ write_node(SerdWriter*        writer,
   return ret;
 }
 
-static inline bool
+static bool
 is_resource(const SerdNode* node)
 {
   return node && node->buf && node->type > SERD_LITERAL;
